import os
import sys
import tempfile
from pathlib import Path

import pytest


@pytest.fixture(autouse=True)
def ensure_clangd_from_wheel(monkeypatch):
    """test the installed clangd package, not the local one"""
    this_dir = Path(__file__).resolve().absolute().parent
    for pd in (this_dir, this_dir / ".."):
        try:
            new_path = sys.path.remove(pd)
            monkeypatch.setattr(sys, "path", new_path)
        except ValueError:
            pass
    monkeypatch.delitem(sys.modules, "clangd", raising=False)


def test_executable_file(capsys):
    import clangd

    clangd._get_executable.cache_clear()
    exe = clangd._get_executable("clangd")
    assert os.path.exists(exe)
    assert os.access(exe, os.X_OK)
    assert capsys.readouterr().out == ""




import subprocess
import sys
from pathlib import Path
import functools
import os

if sys.version_info.minor >= 9:
    # Only available on 3.9 or later, and required on 3.12
    from importlib.resources import files
else:
    import pkg_resources


@functools.lru_cache(maxsize=None)
def _get_executable(name: str) -> Path:
    if sys.version_info.minor >= 9:
        # Only available in 3.9 or later, and required in 3.12
        possibles = [
            Path(files("clangd") / f"data/bin/{name}{s}")
            for s in ("", ".exe", ".bin", ".dmg")
        ]
    else:
        possibles = [
            Path(pkg_resources.resource_filename("clangd", f"data/bin/{name}{s}"))
            for s in ("", ".exe", ".bin", ".dmg")
        ]
    for exe in possibles:
        if exe.exists():
            if os.environ.get("CLANGD_WHEEL_VERBOSE", None):
                print(f"Found binary: {exe} ")
            return exe

    raise FileNotFoundError(f"No executable found for {name} at\n{possibles}")


def _run(name, *args):
    command = [_get_executable(name)]
    if args:
        command += list(args)
    else:
        command += sys.argv[1:]
    return subprocess.call(command)


def _run_python(name, *args):
    command = [sys.executable, _get_executable(name)]
    if args:
        command += list(args)
    else:
        command += sys.argv[1:]

    # as MS Windows is not able to run Python scripts directly by name,
    # we have to call the interpreter and pass the script as parameter
    return subprocess.call(command)


def clangd():
    raise SystemExit(_run("clangd"))




from wheel.bdist_wheel import bdist_wheel as _bdist_wheel
from skbuild import setup

import re

class genericpy_bdist_wheel(_bdist_wheel):
    def finalize_options(self):
        _bdist_wheel.finalize_options(self)
        self.root_is_pure = False

    def get_tag(self):
        python, abi, plat = _bdist_wheel.get_tag(self)
        python, abi = "py2.py3", "none"
        return python, abi, plat

# Read the clangd version from the "single source of truth"
def get_version():
    with open("clangd_version.cmake", "r") as version_file:
        parsed = {}
        for line in version_file:
            match = re.match("set\((.*) (.*)\)", line)
            if len(match.groups()) != 2:
                raise ValueError("Version File not readable")
            parsed[match.groups()[0]] = match.groups()[1]
        if parsed["CLANGD_WHEEL_VERSION"] == "0":
            return f"{parsed['CLANGD_VERSION']}"
        else:
            return f"{parsed['CLANGD_VERSION']}.{parsed['CLANGD_WHEEL_VERSION']}"


# Parse the given README file
with open("README.md", "r") as readme_file:
    readme = readme_file.read()

cmdclass = {"bdist_wheel": genericpy_bdist_wheel}
setup(
    name="clangd",
    version=get_version(),
    cmdclass=cmdclass,
    author="Dan Ilan",
    packages=["clangd"],
    zip_safe=False,
    entry_points={
        "console_scripts": [
            "clangd=clangd:clangd",
        ]
    },
    description="binaries for clangd, a clang-based C++ language server (LSP)",
    long_description=readme,
    long_description_content_type="text/markdown",
    project_urls={
        "Documentation": "https://clangd.llvm.org/",
        "Source": "https://github.com/jmpfar/clangd-wheel",
    },
    keywords=["clangd", "lsp", "language-server", "llvm", "clang", "static-analysis"],
    classifiers=[
        "Environment :: No Input/Output (Daemon)",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: C",
        "Programming Language :: C++",
        "Topic :: Software Development",
    ],
    license="Apache 2.0",
)
